@@ -6,12 +6,17 @@
  *	Copyright (c) 2003-2009, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slony1_funcs.c,v 1.68.2.5 2010-02-16 16:57:48 cbbrowne Exp $
+ *	
  * ----------------------------------------------------------------------
  */
 
 #include "postgres.h"
+#ifdef MSVC
+#include "config_msvc.h"
+#else
 #include "config.h"
+#endif
+
 
 #include "avl_tree.c"
 
@@ -60,6 +65,7 @@ PG_FUNCTION_INFO_V1(_Slony_I_killBackend);
 PG_FUNCTION_INFO_V1(_Slony_I_seqtrack);
 
 PG_FUNCTION_INFO_V1(_slon_quote_ident);
+PG_FUNCTION_INFO_V1(_slon_decode_tgargs);
 
 
 Datum		_Slony_I_createEvent(PG_FUNCTION_ARGS);
@@ -73,6 +79,7 @@ Datum		_Slony_I_killBackend(PG_FUNCTION_ARGS);
 Datum		_Slony_I_seqtrack(PG_FUNCTION_ARGS);
 
 Datum		_slon_quote_ident(PG_FUNCTION_ARGS);
+Datum		_slon_decode_tgargs(PG_FUNCTION_ARGS);
 
 #ifdef CYGWIN
 extern DLLIMPORT Node *newNodeMacroHolder;
@@ -131,7 +138,6 @@ _Slony_I_createEvent(PG_FUNCTION_ARGS)
 	char	   *ev_type_c;
 	Datum		argv[9];
 	char		nulls[10];
-	char	   *buf;
 	size_t		buf_size;
 	int			rc;
 	int			i;
@@ -157,7 +163,6 @@ _Slony_I_createEvent(PG_FUNCTION_ARGS)
 						  PLAN_INSERT_EVENT);
 
 	buf_size = 8192;
-	buf = palloc(buf_size);
 
 	/*
 	 * Do the following only once per transaction.
@@ -1014,7 +1019,7 @@ slon_quote_literal(char *str)
 	char	   *cp1;
 	char	   *cp2;
 	int			len;
-	int			wl;
+	int wl;
 
 	if (str == NULL)
 		return NULL;
@@ -1027,7 +1032,7 @@ slon_quote_literal(char *str)
 	*cp2++ = '\'';
 	while (len > 0)
 	{
-		if ((wl = pg_mblen((unsigned char *) cp1)) != 1)
+		if ((wl = pg_mblen((const char *) cp1)) != 1)
 		{
 			len -= wl;
 
@@ -1182,7 +1187,6 @@ getClusterStatus(Name cluster_name, int need_plan_mask)
 	char		query[1024];
 	bool		isnull;
 	Oid			plan_types[9];
-	Oid			txid_snapshot_typid;
 	TypeName   *txid_snapshot_typname;
 
 	/*
@@ -1267,8 +1271,6 @@ getClusterStatus(Name cluster_name, int need_plan_mask)
 			lappend(lappend(NIL, makeString("pg_catalog")),
 					makeString("txid_snapshot"));
 
-		txid_snapshot_typid = typenameTypeId(NULL, txid_snapshot_typname, NULL);
-
 		/*
 		 * Create the saved plan. We lock the sl_event table in exclusive mode
 		 * in order to ensure that all events are really assigned sequence
@@ -1397,6 +1399,59 @@ getClusterStatus(Name cluster_name, int need_plan_mask)
 }
 
 
+/**
+ * A function to decode the tgargs column of pg_trigger
+ * and return an array of text objects with each trigger
+ * argument.
+ */
+Datum
+_slon_decode_tgargs(PG_FUNCTION_ARGS)
+{
+	const char * arg;
+	size_t elem_size=0;
+	ArrayType * out_array;
+	int idx;
+	bytea	   *t = PG_GETARG_BYTEA_P(0);
+
+	int arg_size = VARSIZE(t)- VARHDRSZ;
+	const char * in_args = VARDATA(t);
+	int array_size = 0;
+	out_array=construct_empty_array(TEXTOID);
+	arg=in_args;
+
+	for(idx = 0; idx < arg_size; idx++)
+	{
+		
+		if(in_args[idx ]=='\0')
+		{
+			text * one_arg = palloc(elem_size+VARHDRSZ);
+			SET_VARSIZE(one_arg,elem_size + VARHDRSZ);
+			memcpy(VARDATA(one_arg),arg,elem_size);
+			out_array = array_set(out_array,
+								  1, &array_size,
+								  PointerGetDatum(one_arg),
+								  false,
+								  -1,
+								  -1,
+								  false , /*typbyval for TEXT*/
+								  'i' /*typalign for TEXT */
+				);
+			elem_size=0;
+			array_size++;
+			arg=&in_args[idx+1];
+		}
+		else
+		{
+			elem_size++;
+		}
+	}
+
+
+	PG_RETURN_ARRAYTYPE_P(out_array);
+}
+	
+	
+	
 /*
  * Local Variables:
  *	tab-width: 4
