@@ -6,7 +6,7 @@
 --	Copyright (c) 2007-2009, PostgreSQL Global Development Group
 --	Author: Jan Wieck, Afilias USA INC.
 --
--- $Id: slony1_funcs.v83.sql,v 1.1.2.3 2010-05-17 17:15:19 ssinger Exp $
+-- 
 -- ----------------------------------------------------------------------
 
 -- ----------------------------------------------------------------------
@@ -26,6 +26,10 @@ declare
 	c_enabled boolean;
 	v_dummy int4;
 begin
+	if not exists (select 1 from pg_catalog.pg_class c, pg_catalog.pg_namespace n 
+                   where c.relname = i_tblname and n.nspname = i_nspname and c.relnamespace = n.oid) then
+        return 'f'::boolean;   -- if table does not exist, then don't vacuum
+	end if;				
 	select 1 into v_dummy from "pg_catalog".pg_settings where name = 'autovacuum' and setting = 'on';
 	if not found then
 		return 't'::boolean;       -- If autovac is turned off, then we gotta vacuum
@@ -52,3 +56,18 @@ end;$$ language plpgsql;
 
 comment on function @NAMESPACE@.ShouldSlonyVacuumTable (name, name) is 
 'returns false if autovacuum handles vacuuming of the table, or if the table does not exist; returns true if Slony-I should manage it';
+
+
+
+
+create or replace function @NAMESPACE@.TruncateOnlyTable ( name) returns void as
+$$
+begin
+	execute 'truncate '|| @NAMESPACE@.slon_quote_input($1);
+end;
+$$
+LANGUAGE plpgsql;
+
+
+comment on function @NAMESPACE@.TruncateOnlyTable(name) is
+'Calls TRUNCATE with out specifying ONLY, syntax supported in version 8.3';
