@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2009, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: cleanup_thread.c,v 1.45.2.4 2010-06-01 15:14:05 ssinger Exp $
+ *	
  *-------------------------------------------------------------------------
  */
 
@@ -15,12 +15,14 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 #include <errno.h>
 #include <signal.h>
-#include <sys/time.h>
 #include <sys/types.h>
+#ifndef WIN32
+#include <sys/time.h>
+#include <unistd.h>
+#endif
 
 #include "slon.h"
 
@@ -143,6 +145,8 @@ cleanupThread_main( /* @unused@ */ void *dummy)
 		{
 			unsigned long latest_xid;
 
+			vac_count = 0;
+
 			latest_xid = get_earliest_xid(dbconn);
 			vacuum_action = "";
 			if (earliest_xid == latest_xid)
