@@ -1,4 +1,4 @@
-/* $Id: confoptions.h,v 1.38 2008-04-23 20:35:43 cbbrowne Exp $ */
+/*  */
 #ifndef _CONFOPTIONS_H_
 #define _CONFOPTIONS_H_
 
@@ -109,11 +109,12 @@ struct config_string
 };
 
 
-
-static struct config_int ConfigureNamesInt[];
-static struct config_bool ConfigureNamesBool[];
-static struct config_real ConfigureNamesReal[];
-static struct config_string ConfigureNamesString[];
+/**
+static struct config_int* ConfigureNamesInt;
+static struct config_bool* ConfigureNamesBool;
+static struct config_real *ConfigureNamesReal;
+static struct config_string*  ConfigureNamesString;
+**/
 #endif
 /*
  * Local Variables:
