@@ -15,13 +15,16 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 #include <errno.h>
 #include <signal.h>
-#include <sys/time.h>
 #include <sys/types.h>
 
+#ifndef WIN32
+#include <unistd.h>
+#include <sys/time.h>
+#endif
+
 #include "slon.h"
 
 
