@@ -6,7 +6,7 @@
  *	Copyright (c) 2003-2009, PostgreSQL Global Development Group
  *	Author: Jan Wieck, Afilias USA INC.
  *
- *	$Id: slony_logshipper.c,v 1.3.2.1 2009-08-17 17:09:59 devrim Exp $
+ *	
  *-------------------------------------------------------------------------
  */
 
@@ -385,6 +385,7 @@ main(int argc, const char *argv[])
 		
 		if (rc < 0)
 		{
+			errlog(LOG_ERROR,"ipc_recv_path returned an error:%d\n",rc);
 			return -1;
 		}
 
