@@ -14,10 +14,14 @@ GetOptions("config=s" => \$CONFIG_FILE,
 	   "help"     => \$SHOW_USAGE);
 
 my $USAGE =
-"Usage: drop_node [--config file] node#
+"Usage: drop_node [--config file] node# event_node#
 
     Drops a node.
 
+node# is the id of the node in config file you wish to remove from cluster.
+
+event_node# is the node number of the current origin node; the one capable of
+sending notification to other nodes.
 This will lead to Slony-I dropping the triggers (generally that deny the
 ability to update data), restoring the \"native\" triggers, dropping the
 schema used by Slony-I, and the slon process for that node terminating
@@ -39,21 +43,28 @@ if ($SHOW_USAGE) {
   exit 0;
 }
 
-require '@@PGLIBDIR@@/slon-tools.pm';
+require '@@PERLSHAREDIR@@/slon-tools.pm';
 require $CONFIG_FILE;
 
-my ($node) = @ARGV;
+my ($node, $event_node) = @ARGV;
 if ($node =~ /^(?:node)?(\d+)$/) {
   $node = $1;
 } else {
   die $USAGE;
 }
 
+if ($event_node =~ /^(?:node)?(\d+)$/) {
+  $event_node = $1;
+} else {
+  print "Need to specify event node!\n";
+  die $USAGE;
+}
+
 my $slonik = '';
 
 $slonik .= genheader();
 $slonik .= "  try {\n";
-$slonik .= "      drop node (id = $node, event node = $MASTERNODE);\n";
+$slonik .= "      drop node (id = $node, event node = $event_node);\n";
 $slonik .= "  } on error {\n";
 $slonik .= "      echo 'Failed to drop node $node from cluster';\n";
 $slonik .= "      exit 1;\n";
