@@ -72,8 +72,11 @@ report_on_problems ();
 
 sub test_node {
   my ($node, $dsn) = @_;
+  $dsn="dbi:Pg:$dsn";
 
   print "\nTests for node $node - DSN = $dsn\n========================================\n";
+  $dbh = DBI->connect($dsn) or add_problem($node, "Could not connect.",
+					   qq(Could not connect to node $node using DSN: $dsn)) and return;
 
   my $listener_query = "select relpages, reltuples from pg_catalog.pg_class where relname = 'pg_listener';";
   my $res = $dbh->prepare($listener_query);
@@ -261,7 +264,7 @@ Could listen paths be missing so that confirmations are not propagating?
   print "\n";
 
   print "\n------------------------------------------------------------------------------\n";
-  print "\nListing of old open connections\n";
+  printf "\nListing of old open connections on node %d\n", $node;
   printf "%15s %15s %15s %12s %20s\n", "Database", "PID", "User", "Query Age", "Query";
   print "================================================================================\n";
 
